from dataclasses import dataclass


@dataclass(frozen=True)
class TransformationParameters:
    TRANSFORMATION_TYPE = "transformation_type"
    LOW = "low"
    HIGH = "high"
    K = "k"
    COEF_DIV = "coef_div"
    COEF_SI = "coef_si"
    COEF_SE = "coef_se"
    TRUNCATE_LEFT = "truncate_left"
    TRUNCATE_RIGHT = "truncate_right"
    INTERPOLATION_MAP = "interpolation_map"


TransformationParametersEnum = TransformationParameters()



import json
import os
from typing import List

import numpy as np
from reinvent_chemistry.library_design.aizynth.aizynth_client import AiZynthClient

from reinvent_scoring.scoring.component_parameters import ComponentParameters
from reinvent_scoring.scoring.enums.environmental_variables_enum import EnvironmentalVariablesEnum
from reinvent_scoring.scoring.score_components.base_score_component import BaseScoreComponent
from reinvent_scoring.scoring.score_summary import ComponentSummary


class MockedLogger:
    def log_message(self, message: str):
        print(message)

class BuildingBlockAvailabilityComponent(BaseScoreComponent):
    def __init__(self, parameters: ComponentParameters):
        super().__init__(parameters)
        self._environment_keys = EnvironmentalVariablesEnum()
        self._aizynth_cient = self._set_up_client()

    def calculate_score(self, molecules: List, step=-1) -> ComponentSummary:
        valid_smiles = self._chemistry.mols_to_smiles(molecules)
        score = self._score_smiles(valid_smiles)
        score_summary = ComponentSummary(total_score=score, parameters=self.parameters, raw_score=score)

        return score_summary

    def _score_smiles(self, smiles: List[str]) -> np.array:
        list_of_pathways = self._aizynth_cient.batch_synthesis_prediction(smiles)
        results = self._aizynth_cient.batch_stock_availability_score(list_of_pathways)
        return np.array(results)

    def _set_up_client(self):
        logger = MockedLogger()
        prediction_url = self._get_enviornment_variable(self._environment_keys.AIZYNTH_PREDICTION_URL)
        availability_url = self._get_enviornment_variable(self._environment_keys.AIZYNTH_BUILDING_BLOCKS_URL)
        api_token = self._get_enviornment_variable(self._environment_keys.AIZYNTH_TOKEN)
        aizynth_cient = AiZynthClient(prediction_url, availability_url, api_token, logger)
        return aizynth_cient

    def _get_enviornment_variable(self, variable: str) -> str:
        try:
            return os.environ[variable]
        except KeyError:
            return self._retrieve_aizynth_key_from_config(variable)

    def _retrieve_aizynth_key_from_config(self, variable: str) -> str:
        try:
            project_root = os.path.dirname(__file__)
            with open(os.path.join(project_root, '../../../configs/config.json'), 'r') as f:
                config = json.load(f)
            environmental_variables = config[self._environment_keys.ENVIRONMENTAL_VARIABLES]
            return environmental_variables[variable]
        except KeyError as ex:
            raise KeyError(f"Key {variable} not found in reinvent_scoring/configs/config.json")


class EnvironmentalVariablesEnum:
    PIP_URL = "PIP_URL"
    PIP_KEY = "PIP_KEY"
    PIP_GET_RESULTS = "PIP_GET_RESULTS"
    ENVIRONMENTAL_VARIABLES = "ENVIRONMENTAL_VARIABLES"
    AIZYNTH_PREDICTION_URL = "AIZYNTH_PREDICTION_URL"
    AIZYNTH_BUILDING_BLOCKS_URL = "AIZYNTH_BUILDING_BLOCKS_URL"
    AIZYNTH_TOKEN = "AIZYNTH_TOKEN"



from dataclasses import dataclass


@dataclass(frozen=True)
class ComponentSpecificParametersEnum:
    SCIKIT = "scikit"
    CLAB_INPUT_FILE = "clab_input_file"
    DESCRIPTOR_TYPE = "descriptor_type"
    TRANSFORMATION = "transformation"

    # structural components
    # ---------
    # AZDOCK
    AZDOCK_CONFPATH = "configuration_path"
    AZDOCK_DOCKERSCRIPTPATH = "docker_script_path"
    AZDOCK_ENVPATH = "environment_path"
    AZDOCK_DEBUG = "debug"

    # DockStream
    DOCKSTREAM_CONFPATH = "configuration_path"
    DOCKSTREAM_DOCKERSCRIPTPATH = "docker_script_path"
    DOCKSTREAM_ENVPATH = "environment_path"
    DOCKSTREAM_DEBUG = "debug"

    # AZGARD
    AZGARD_CONFPATH = "configuration_path"
    AZGARD_EXECUTORSCRIPTPATH = "executor_script_path"
    AZGARD_ENVPATH = "environment_path"
    AZGARD_VALUES_KEY = "values_key"
    AZGARD_DEBUG = "debug"
    #######################

    RAT_PK_PROPERTY = "rat_pk_property"
    CLAB_TOP_20_VALUE = "clab_top_20_value"
    ION_CLASS = "Ion class"
    CONTAINER_TYPE = "container_type"

    SMILES = "smiles"
    MODEL_PATH = "model_path"
